@@ -45,6 +45,9 @@
 #include <qmath.h>
 #include "qquickstyledtext_p.h"
 #include <QQmlContext>
+#include <QtGui/private/qtexthtmlparser_p.h>
+
+Q_LOGGING_CATEGORY(lcStyledText, "qt.quick.styledtext")
 
 /*
     QQuickStyledText supports few tags:
@@ -89,8 +92,8 @@ public:
                                   QQmlContext *context,
                                   bool preloadImages,
                                   bool *fontSizeModified)
-        : text(t), layout(l), imgTags(&imgTags), baseFont(layout.font()), baseUrl(baseUrl), hasNewLine(true), nbImages(0), updateImagePositions(false)
-        , preFormat(false), prependSpace(false), hasSpace(true), preloadImages(preloadImages), fontSizeModified(fontSizeModified), context(context)
+        : text(t), layout(l), imgTags(&imgTags), baseFont(layout.font()), baseUrl(baseUrl),
+          fontSizeModified(fontSizeModified), context(context), preloadImages(preloadImages)
     {
     }
 
@@ -104,8 +107,8 @@ public:
     bool parseUnorderedListAttributes(const QChar *&ch, const QString &textIn);
     bool parseAnchorAttributes(const QChar *&ch, const QString &textIn, QTextCharFormat &format);
     void parseImageAttributes(const QChar *&ch, const QString &textIn, QString &textOut);
-    QPair<QStringRef,QStringRef> parseAttribute(const QChar *&ch, const QString &textIn);
-    QStringRef parseValue(const QChar *&ch, const QString &textIn);
+    QPair<QStringView,QStringView> parseAttribute(const QChar *&ch, const QString &textIn);
+    QStringView parseValue(const QChar *&ch, const QString &textIn);
     void setFontSize(int size, QTextCharFormat &format);
 
     inline void skipSpace(const QChar *&ch) {
@@ -122,15 +125,15 @@ public:
     QFont baseFont;
     QStack<List> listStack;
     QUrl baseUrl;
-    bool hasNewLine;
-    int nbImages;
-    bool updateImagePositions;
-    bool preFormat;
-    bool prependSpace;
-    bool hasSpace;
-    bool preloadImages;
     bool *fontSizeModified;
     QQmlContext *context;
+    int nbImages = 0;
+    bool hasNewLine = true;
+    bool updateImagePositions = false;
+    bool preFormat = false;
+    bool prependSpace = false;
+    bool hasSpace = true;
+    bool preloadImages;
 
     static const QChar lessThan;
     static const QChar greaterThan;
@@ -298,7 +301,7 @@ void QQuickStyledTextPrivate::appendText(const QString &textIn, int start, int l
 {
     if (prependSpace)
         textOut.append(space);
-    textOut.append(QStringRef(&textIn, start, length));
+    textOut.append(QStringView(textIn).mid(start, length));
     prependSpace = false;
     hasSpace = false;
     hasNewLine = false;
@@ -328,7 +331,7 @@ bool QQuickStyledTextPrivate::parseTag(const QChar *&ch, const QString &textIn,
         if (*ch == greaterThan) {
             if (tagLength == 0)
                 return false;
-            QStringRef tag(&textIn, tagStart, tagLength);
+            auto tag = QStringView(textIn).mid(tagStart, tagLength);
             const QChar char0 = tag.at(0);
             if (char0 == QLatin1Char('b')) {
                 if (tagLength == 1) {
@@ -355,7 +358,7 @@ bool QQuickStyledTextPrivate::parseTag(const QChar *&ch, const QString &textIn,
                     preFormat = true;
                     if (!hasNewLine)
                         textOut.append(QChar::LineSeparator);
-                    format.setFontFamily(QString::fromLatin1("Courier New,courier"));
+                    format.setFontFamilies(QStringList {QString::fromLatin1("Courier New"), QString::fromLatin1("courier")});
                     format.setFontFixedPitch(true);
                     return true;
                 }
@@ -437,7 +440,7 @@ bool QQuickStyledTextPrivate::parseTag(const QChar *&ch, const QString &textIn,
             return false;
         } else if (ch->isSpace()) {
             // may have params.
-            QStringRef tag(&textIn, tagStart, tagLength);
+            auto tag = QStringView(textIn).mid(tagStart, tagLength);
             if (tag == QLatin1String("font"))
                 return parseFontAttributes(ch, textIn, format);
             if (tag == QLatin1String("ol")) {
@@ -475,7 +478,7 @@ bool QQuickStyledTextPrivate::parseCloseTag(const QChar *&ch, const QString &tex
         if (*ch == greaterThan) {
             if (tagLength == 0)
                 return false;
-            QStringRef tag(&textIn, tagStart, tagLength);
+            auto tag = QStringView(textIn).mid(tagStart, tagLength);
             const QChar char0 = tag.at(0);
             hasNewLine = false;
             if (char0 == QLatin1Char('b')) {
@@ -555,20 +558,15 @@ void QQuickStyledTextPrivate::parseEntity(const QChar *&ch, const QString &textI
     int entityLength = 0;
     while (!ch->isNull()) {
         if (*ch == QLatin1Char(';')) {
-            QStringRef entity(&textIn, entityStart, entityLength);
-            if (entity == QLatin1String("gt"))
-                textOut += QChar(62);
-            else if (entity == QLatin1String("lt"))
-                textOut += QChar(60);
-            else if (entity == QLatin1String("amp"))
-                textOut += QChar(38);
-            else if (entity == QLatin1String("quot"))
-                textOut += QChar(34);
-            else if (entity == QLatin1String("nbsp"))
-                textOut += QChar(QChar::Nbsp);
+            auto entity = QStringView(textIn).mid(entityStart, entityLength);
+            const QString parsedEntity = QTextHtmlParser::parseEntity(entity);
+            if (!parsedEntity.isNull())
+                textOut += parsedEntity;
+            else
+                qCWarning(lcStyledText) << "StyledText doesn't support entity" << entity;
             return;
         } else if (*ch == QLatin1Char(' ')) {
-            QStringRef entity(&textIn, entityStart - 1, entityLength + 1);
+            auto entity = QStringView(textIn).mid(entityStart - 1, entityLength + 1);
             textOut += entity + *ch;
             return;
         }
@@ -580,7 +578,7 @@ void QQuickStyledTextPrivate::parseEntity(const QChar *&ch, const QString &textI
 bool QQuickStyledTextPrivate::parseFontAttributes(const QChar *&ch, const QString &textIn, QTextCharFormat &format)
 {
     bool valid = false;
-    QPair<QStringRef,QStringRef> attr;
+    QPair<QStringView,QStringView> attr;
     do {
         attr = parseAttribute(ch, textIn);
         if (attr.first == QLatin1String("color")) {
@@ -608,7 +606,7 @@ bool QQuickStyledTextPrivate::parseOrderedListAttributes(const QChar *&ch, const
     listItem.type = Ordered;
     listItem.format = Decimal;
 
-    QPair<QStringRef,QStringRef> attr;
+    QPair<QStringView,QStringView> attr;
     do {
         attr = parseAttribute(ch, textIn);
         if (attr.first == QLatin1String("type")) {
@@ -637,7 +635,7 @@ bool QQuickStyledTextPrivate::parseUnorderedListAttributes(const QChar *&ch, con
     listItem.type = Unordered;
     listItem.format = Bullet;
 
-    QPair<QStringRef,QStringRef> attr;
+    QPair<QStringView,QStringView> attr;
     do {
         attr = parseAttribute(ch, textIn);
         if (attr.first == QLatin1String("type")) {
@@ -657,7 +655,7 @@ bool QQuickStyledTextPrivate::parseAnchorAttributes(const QChar *&ch, const QStr
 {
     bool valid = false;
 
-    QPair<QStringRef,QStringRef> attr;
+    QPair<QStringView,QStringView> attr;
     do {
         attr = parseAttribute(ch, textIn);
         if (attr.first == QLatin1String("href")) {
@@ -682,7 +680,7 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         QQuickStyledTextImgTag *image = new QQuickStyledTextImgTag;
         image->position = textOut.length() + (trailingSpace ? 0 : 1);
 
-        QPair<QStringRef,QStringRef> attr;
+        QPair<QStringView,QStringView> attr;
         do {
             attr = parseAttribute(ch, textIn);
             if (attr.first == QLatin1String("src")) {
@@ -706,7 +704,7 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
             // to avoid a relayout later on.
             QUrl url = baseUrl.resolved(image->url);
             if (url.isLocalFile()) {
-                image->pix = new QQuickPixmap(context->engine(), url, image->size);
+                image->pix = new QQuickPixmap(context->engine(), url, QRect(), image->size);
                 if (image->pix && image->pix->isReady()) {
                     image->size = image->pix->implicitSize();
                 } else {
@@ -727,7 +725,7 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         image->position = textOut.length() + (trailingSpace ? 0 : 1);
         imgWidth = image->size.width();
         image->offset = -std::fmod(imgWidth, spaceWidth) / 2.0;
-        QPair<QStringRef,QStringRef> attr;
+        QPair<QStringView,QStringView> attr;
         do {
             attr = parseAttribute(ch, textIn);
         } while (!ch->isNull() && !attr.first.isEmpty());
@@ -740,7 +738,7 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
     textOut += padding + QLatin1Char(' ');
 }
 
-QPair<QStringRef,QStringRef> QQuickStyledTextPrivate::parseAttribute(const QChar *&ch, const QString &textIn)
+QPair<QStringView,QStringView> QQuickStyledTextPrivate::parseAttribute(const QChar *&ch, const QString &textIn)
 {
     skipSpace(ch);
 
@@ -759,10 +757,10 @@ QPair<QStringRef,QStringRef> QQuickStyledTextPrivate::parseAttribute(const QChar
             ++ch;
             if (!attrLength)
                 break;
-            QStringRef attr(&textIn, attrStart, attrLength);
-            QStringRef val = parseValue(ch, textIn);
+            auto attr = QStringView(textIn).mid(attrStart, attrLength);
+            QStringView val = parseValue(ch, textIn);
             if (!val.isEmpty())
-                return QPair<QStringRef,QStringRef>(attr,val);
+                return QPair<QStringView,QStringView>(attr,val);
             break;
         } else {
             ++attrLength;
@@ -770,10 +768,10 @@ QPair<QStringRef,QStringRef> QQuickStyledTextPrivate::parseAttribute(const QChar
         ++ch;
     }
 
-    return QPair<QStringRef,QStringRef>();
+    return QPair<QStringView,QStringView>();
 }
 
-QStringRef QQuickStyledTextPrivate::parseValue(const QChar *&ch, const QString &textIn)
+QStringView QQuickStyledTextPrivate::parseValue(const QChar *&ch, const QString &textIn)
 {
     int valStart = ch - textIn.constData();
     int valLength = 0;
@@ -782,10 +780,10 @@ QStringRef QQuickStyledTextPrivate::parseValue(const QChar *&ch, const QString &
         ++ch;
     }
     if (ch->isNull())
-        return QStringRef();
+        return QStringView();
     ++ch; // skip quote
 
-    return QStringRef(&textIn, valStart, valLength);
+    return QStringView(textIn).mid(valStart, valLength);
 }
 
 QString QQuickStyledTextPrivate::toAlpha(int value, bool upper)
